<?php

/**
 * Nexi XPay Payment Module
 *
 * @author      iPlusService S.r.l. <asssistenza@iplusservice.it>
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     4.0.0
 * @copyright   2017-2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */
class Iplusservice_XPay_Block_Info extends Mage_Payment_Block_Info
{

    private function getCorrectTemplate()
    {
        $order = $this->getMethod()->getInfoInstance()->getOrder();

        if ($order instanceof Mage_Sales_Model_Order) {
            $payment = $order->getPayment();

            if ($payment instanceof Mage_Sales_Model_Order_Payment) {
                if ($payment->getAdditionalInformation('alias') && $payment->getAdditionalInformation('transactionId')) {
                    $template = 'iplusservice/xpay/info/default.phtml';
                } else {
                    if ($order->getBaseTotalDue() > 0) {
                        $template = 'iplusservice/xpay/info/paymail.phtml';
                    } else {
                        $template = 'iplusservice/xpay/info/title.phtml';
                    }
                }
            } else {
                $template = 'iplusservice/xpay/info/title.phtml';
            }
        } else {
            $template = 'iplusservice/xpay/info/title.phtml';
        }

        return $template;
    }

    protected function _toHtml()
    {
        $this->setTemplate($this->getCorrectTemplate());

        if (!$this->getTemplate()) {
            return '';
        } else {
            return $this->renderView();
        }
    }

    public function getPaymailUrl()
    {
        $method = $this->getMethod();
        $order = $method->getInfoInstance()->getOrder();

        if ($order instanceof Mage_Sales_Model_Order) {
            $payment = $order->getPayment();

            if ($payment instanceof Mage_Sales_Model_Order_Payment) {
                $helper = Mage::helper('iplusservice_xpay');

                $configuration = Mage::getModel('iplusservice_xpay/configuration', $method);

                $api = Mage::getModel('iplusservice_xpay/api', $configuration);

                $transactionId = $helper->createTransactionId($order);

                $payment->setAdditionalInformation('alias', $configuration->getAlias());
                $payment->setAdditionalInformation('transactionId', $transactionId);
                $payment->setAdditionalInformation('paymail', 1);

                $payment->save();

                $amount = $helper->getGrandTotalFormatted($order);
                $url = Mage::getUrl('xpay/payment/conferma', array('_secure' => true));
                $urlPost = Mage::getUrl('xpay/payment/notificas2s', array('_secure' => true));
                $urlBack = Mage::getUrl('xpay/payment/annullo', array('_secure' => true));

                return $api->getPayMailLink($transactionId, $amount, $url, $urlPost, $urlBack, null, 72);
            }
        }

        return false;
    }

    public function getTransactionInfo()
    {
        $configuration = Mage::getModel('iplusservice_xpay/configuration', $this->getMethod());

        $api = Mage::getModel('iplusservice_xpay/api', $configuration);

        try {
            $response = $api->getTransactionInfo(
                    $this->getInfo()->getAdditionalInformation('alias'),
                    $this->getInfo()->getAdditionalInformation('transactionId')
            );

            $info = array(
                'card_holder' => array(
                    $this->__('Name') => $response['dettaglio'][0]['nome'] . ' ' . $response['dettaglio'][0]['cognome'],
                    'Email' => $response['dettaglio'][0]['mail']
                ),
                'payment_method' => array(
                    $this->__('Method') => $response['brand'],
                    $this->__('Nationality') => $response['nazione']
                ),
                'transaction_details' => array(
                    $this->__('Transaction date') => $this->formatXPayDate($response['dataTransazione']),
                    $this->__('Transaction status') => $response['stato'],
                    $this->__('Amount') => $this->formatAmount($response['importo']) . ' ' . $response['divisa'],
                    $this->__('Transaction code') => $response['codiceTransazione']
                ),
                'operations' => $response['dettaglio'][0]['operazioni']
            );

            if ($response['pan']) {
                $info['payment_method']['Pan'] = $response['pan'];
            }

            if ($response['scadenza']) {
                $info['payment_method'][$this->__('Expiration date')] = substr($response['scadenza'], -2) . '/' . substr($response['scadenza'], 0, 4);
            }

            foreach ($info as $block => $values) {
                if ($block != 'operations') {
                    $blockInvalid = true;

                    foreach ($values as $name => $value) {
                        if (!trim($value)) {
                            unset($info[$block][$name]);
                        } else {
                            $blockInvalid = false;
                        }
                    }

                    if ($blockInvalid) {
                        unset($info[$block]);
                    }
                }
            }
        } catch (Exception $ex) {
            error_log(__METHOD__ . '::' . __LINE__ . ' ' . $ex->getMessage() . ' ' . $ex->getTraceAsString());

            $info = array();
        }

        return $info;
    }

    public function formatAmount($amount)
    {
        return number_format($amount / 100, 2, ",", " ");
    }

    public function formatXPayDate($date, $format = 'd/m/Y H:i:s')
    {
        try {
            $oDate = new DateTime($date);

            return $oDate->format($format);
        } catch (Exception $ex) {
            
        }

        return '';
    }

}
