<?php

/**
 * Nexi XPay Payment Module
 *
 * @author      iPlusService S.r.l. <asssistenza@iplusservice.it>
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     4.0.0
 * @copyright   2017-2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */
class Iplusservice_XPay_Helper_Data extends Mage_Payment_Helper_Data
{

    public function getDefaultLogo()
    {
        return Mage::getBaseUrl('media', array('_secure' => true)) . 'Iplusservice/XPay/images/logo.jpg';
    }

    private static function getBrandImageMapping()
    {
        return array(
            'visa' => 'visa.png',
            'mastercard' => 'mastercard.png',
            'maestro' => 'maestro.png',
            'amex' => 'amex.png',
            'diners' => 'diners.jpg',
            'jcb' => 'jcb.gif'
        );
    }

    public function brandExists($brand)
    {
        return array_key_exists(strtolower(trim($brand)), self::getBrandImageMapping());
    }

    public function fromBrandToImage($brand)
    {
        $imagePaths = self::getBrandImageMapping();

        if (array_key_exists(strtolower(trim($brand)), $imagePaths)) {
            return Mage::getBaseUrl('media', array('_secure' => true)) . 'Iplusservice/XPay/images/' . $imagePaths[strtolower(trim($brand))];
        }

        return '';
    }

    public function getPrimaryLogoStyle($brand)
    {
        $configuration = array(
            'visa' => 'height:18px; margin-top:5px;',
            'maestro' => 'height:30px;',
            'mastercard' => 'height:30px;',
            'diners' => 'height:25px; margin-top:2px;',
            'amex' => 'height:29px;',
            'jcb' => 'height:25px; margin-top:2px;'
        );

        return array_key_exists(strtolower(trim($brand)), $configuration) ? $configuration[strtolower(trim($brand))] : 'height:25px; margin-top:2px;';
    }

    public function getSecondaryLogoStyle($brand)
    {
        $configuration = array(
            'visa' => 'height:20px; margin-top:5px;',
            'maestro' => 'height:30px;',
            'mastercard' => 'height:30px;',
            'diners' => 'height:30px;',
            'amex' => 'height:30px;',
            'jcb' => 'height:30px;'
        );

        return array_key_exists(strtolower(trim($brand)), $configuration) ? $configuration[strtolower(trim($brand))] : 'height:30px;';
    }

    public function pluginVersion()
    {
        return '3.1.0';
    }

    public function createTransactionId($order)
    {
        return substr($order->getId() . '-' . time(), 0, 30);
    }

    public function getGrandTotalFormatted($order)
    {
        $amount = $order->getGrandTotal();

        $currencyCode = Mage::app()->getStore()->getCurrentCurrencyCode();
        $divisa = 'EUR';

        if ($currencyCode != $divisa) {
            $amount = Mage::helper('directory')->currencyConvert($amount, $currencyCode, $divisa);
        }

        return strtr(round($amount * 100), array('.' => '', ',' => ''));
    }

    public function getLanguageId($paymentMethod)
    {
        if ($paymentMethod->getConfigData('language') == "AUTO") {
            $locale = Mage::app()->getLocale()->getLocaleCode();

            $aLang = array(
                'it_IT' => 'ITA', 'it_CH' => 'ITA',
                'en_AU' => 'ENG', 'en_CA' => 'ENG', 'en_IE' => 'ENG', 'en_NZ' => 'ENG', 'en_GB' => 'ENG', 'en_US' => 'ENG',
                'es_AR' => 'SPA', 'es_CL' => 'SPA', 'es_CO' => 'SPA', 'es_CR' => 'SPA', 'es_MX' => 'SPA', 'es_PA' => 'SPA', 'es_PE' => 'SPA', 'es_ES' => 'SPA', 'es_VE' => 'SPA',
                'fr_CA' => 'FRA', 'fr_FR' => 'FRA',
                'de_AT' => 'GER', 'de_DE' => 'GER', 'de_CH' => 'GER',
                'ja_JP' => 'JPN',
                'pt_PT' => 'POR'
            );

            if (isset($aLang[$locale])) {
                $languageId = $aLang[$locale];
            } else {
                $languageId = 'ENG';
            }
        } else {
            $languageId = $paymentMethod->getConfigData('language');
        }

        return trim($languageId);
    }

    public function invoiceOrder(Mage_Sales_Model_Order $order, $transactionId)
    {
        try {
            if (!$order->canInvoice()) {
                Mage::throwException(Mage::helper('core')->__('Cannot create an invoice.'));
            }

            $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();

            if (!$invoice->getTotalQty()) {
                Mage::throwException(Mage::helper('core')->__('Cannot create an invoice without products.'));
            }

            $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE);
            $invoice->register();
            $invoice->setTransactionId($transactionId);
            $invoice->pay();

            $transactionSave = Mage::getModel('core/resource_transaction')
                    ->addObject($invoice)
                    ->addObject($invoice->getOrder());

            $transactionSave->save();
        } catch (Mage_Core_Exception $e) {
            
        }
    }

}
