<?php

/**
 * Nexi XPay Payment Module
 *
 * @author      iPlusService S.r.l. <asssistenza@iplusservice.it>
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     4.0.0
 * @copyright   2017-2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */
class Iplusservice_XPay_Model_Api
{

    /**
     *
     * @var Iplusservice_XPay_Model_Configuration
     */
    private $configuration;

    /**
     *
     * @var Iplusservice_XPay_Model_Gateway
     */
    private $gateway;

    /**
     *
     * @param Iplusservice_XPay_Model_Configuration $configuration
     */
    public function __construct(Iplusservice_XPay_Model_Configuration $configuration)
    {
        $this->configuration = $configuration;

        $this->gateway = Mage::getModel('iplusservice_xpay/gateway');
    }

    /**
     *
     * @param string $transactionAlias
     * @param string $transactionId
     * @param float $amount
     * @throws Exception
     */
    public function refund($transactionAlias, $transactionId, $amount)
    {
        list($alias, $mac) = $this->getCredentials($transactionAlias);

        $currency = $this->gateway->CURRENCY_NUM_EUR;
        $timestamp = time() * 1000;

        $requestMac = sha1(
                'apiKey=' . $alias
                . 'codiceTransazione=' . $transactionId
                . 'divisa=' . $currency
                . 'importo=' . ((int) (string) $amount)
                . 'timeStamp=' . $timestamp
                . $mac
        );

        $request = array(
            'apiKey' => $alias,
            'codiceTransazione' => $transactionId,
            'importo' => ((int) (string) $amount),
            'divisa' => $currency,
            'timeStamp' => $timestamp,
            'mac' => $requestMac,
        );

        try {
            $response = $this->execCurl($this->gateway->URI_REFUND, $request);
        } catch (Exception $ex) {
            throw new Exception('Error during refund', 500, $ex);
        }

        $responseMac = sha1(
                'esito=' . $response['esito']
                . 'idOperazione=' . $response['idOperazione']
                . 'timeStamp=' . $response['timeStamp']
                . $mac
        );

        if ($responseMac !== $response['mac']) {
            throw new Exception('Invalid response mac', 501);
        }

        if (strtoupper(trim($response['esito'])) !== "OK") {
            throw new Exception('Error during refund', 502);
        }
    }

    /**
     *
     * @param string $transactionAlias
     * @param string $transactionId
     * @return array
     * @throws Exception
     */
    public function getTransactionInfo($transactionAlias, $transactionId)
    {
        list($alias, $mac) = $this->getCredentials($transactionAlias);

        $timestamp = time() * 1000;

        $requestMac = sha1(
                'apiKey=' . $alias
                . 'codiceTransazione=' . $transactionId
                . 'timeStamp=' . $timestamp
                . $mac
        );

        $request = array(
            'apiKey' => $alias,
            'codiceTransazione' => $transactionId,
            'timeStamp' => $timestamp,
            'mac' => $requestMac
        );

        try {
            $response = $this->execCurl($this->gateway->URI_TRANSACTION_INFO, $request);
        } catch (Exception $ex) {
            throw new Exception('Error during API call', 500, $ex);
        }

        $responseMac = sha1(
                'esito=' . $response['esito']
                . 'idOperazione=' . $response['idOperazione']
                . 'timeStamp=' . $response['timeStamp']
                . $mac
        );

        if ($responseMac !== $response['mac']) {
            throw new Exception('Invalid response mac', 501);
        }

        if (strtoupper(trim($response['esito'])) !== "OK") {
            throw new Exception('Error during information retrieval', 502);
        }

        return $response['report'][0];
    }

    public function getPayMailLink($transactionId, $amount, $url, $urlPost, $urlBack, $mail = null, $timeout = 72)
    {
        $timestamp = time() * 1000;

        $requestMac = sha1(
                'apiKey=' . $this->configuration->getAlias()
                . 'codiceTransazione=' . $transactionId
                . 'importo=' . ((int) (string) $amount)
                . 'timeStamp=' . $timestamp
                . $this->configuration->getMac()
        );

        $request = array(
            'apiKey' => $this->configuration->getAlias(),
            'url' => $url,
            'codiceTransazione' => $transactionId,
            'importo' => ((int) (string) $amount),
            'timeout' => $timeout,
            'timeStamp' => $timestamp,
            'mac' => $requestMac,
            'parametriAggiuntivi' => array(
                'TCONTAB' => $this->configuration->getAccountingType(),
                'urlpost' => $urlPost,
                'url_back' => $urlBack,
                'mail' => $mail,
                'Note1' => 'magento',
                'Note2' => '1.9',
                'Note3' => Mage::helper('iplusservice_xpay')->pluginVersion()
            )
        );

        try {
            $response = $this->execCurl($this->gateway->URI_PAYMAIL, $request);
        } catch (Exception $ex) {
            throw new Exception($ex->getMessage(), 500, $ex);
        }

        $responseMac = sha1(
                'esito=' . $response['esito']
                . 'idOperazione=' . $response['idOperazione']
                . 'timeStamp=' . $response['timeStamp']
                . $this->configuration->getMac()
        );

        if ($responseMac !== $response['mac']) {
            throw new Exception('Response: Invalid MAC', 501);
        }

        if (strtoupper(trim($response['esito'])) !== "OK") {
            throw new Exception('Error 2', 502);
        }

        return $response['payMailUrl'];
    }

    public function getAvailableMethods()
    {
        $timestamp = time() * 1000;

        $requestMac = sha1(
                'apiKey=' . $this->configuration->getAlias()
                . 'timeStamp=' . $timestamp
                . $this->configuration->getMac()
        );

        $request = array(
            'apiKey' => $this->configuration->getAlias(),
            'timeStamp' => $timestamp,
            'platform' => 'magento',
            'platformVers' => '1.9',
            'pluginVers' => Mage::helper('iplusservice_xpay')->pluginVersion(),
            'mac' => $requestMac
        );

        try {
            $response = $this->execCurl($this->gateway->URI_PROFILE_INFO, $request);
        } catch (Exception $ex) {
            throw new Exception($ex->getMessage(), 500, $ex);
        }

        $responseMac = sha1(
                'esito=' . $response['esito']
                . 'idOperazione=' . $response['idOperazione']
                . 'timeStamp=' . $response['timeStamp']
                . $this->configuration->getMac()
        );

        if (false && $responseMac !== $response['mac']) {
            throw new Exception('Response: Invalid MAC', 501);
        }

        if (strtoupper(trim($response['esito'])) !== "OK") {
            throw new Exception('Error 2', 502);
        }

        $result = array(
            'urlLogo' => $response['urlLogoNexiLarge'],
            'availableMethods' => $response['availableMethods']
        );

        return $result;
    }

    /**
     *
     * @param string $transactionAlias
     * @return array
     */
    private function getCredentials($transactionAlias)
    {
        $profiles = array(
            $this->configuration->getAlias() => $this->configuration->getMac(),
            $this->configuration->getAliasOneClick() => $this->configuration->getMacOneClick()
        );

        if (!array_key_exists(trim($transactionAlias), $profiles)) {
            throw new Exception('Profile not found');
        }

        return array(
            $transactionAlias,
            $profiles[trim($transactionAlias)]
        );
    }

    private function execCurl($uri, $request)
    {
        $url = strpos($uri, 'https') === false ? $this->configuration->getUrl() . $uri : $uri;

        $connection = curl_init();

        if ($connection === false) {
            throw new Exception('Error curl init');
        }

        curl_setopt_array($connection, array(
            CURLOPT_URL => $url,
            CURLOPT_HTTPHEADER => array('Content-Type: application/json'),
            CURLOPT_POST => 1,
            CURLOPT_POSTFIELDS => json_encode($request),
            CURLOPT_RETURNTRANSFER => 1,
            CURLINFO_HEADER_OUT => true,
            CURLOPT_SSL_VERIFYPEER => 0
        ));

        $responseContent = curl_exec($connection);

        if (curl_error($connection)) {
            error_log(__METHOD__ . '::' . __LINE__ . ' ' . curl_error($connection));
        }

        curl_close($connection);

        if ($responseContent === false) {
            throw new Exception('Failed curl 2');
        }

        $response = json_decode($responseContent, true);

        if (json_last_error() !== JSON_ERROR_NONE) {
            throw new Exception('Malformed JSON response 3');
        }

        return $response;
    }

}
