<?php

/**
 * Nexi XPay Payment Module
 *
 * @author      iPlusService S.r.l. <asssistenza@iplusservice.it>
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     4.0.0
 * @copyright   2017-2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */
class Iplusservice_XPay_Model_Configuration
{

    /**
     *
     * @var Mage_Payment_Model_Method_Abstract
     */
    private $paymentMethod;

    /**
     *
     * @var Iplusservice_XPay_Model_Gateway
     */
    private $gateway;

    /**
     *
     * @param Mage_Payment_Model_Method_Abstract $paymentMethod
     */
    public function __construct(Mage_Payment_Model_Method_Abstract $paymentMethod)
    {
        $this->paymentMethod = $paymentMethod;

        $this->gateway = Mage::getModel('iplusservice_xpay/gateway');
    }

    /**
     *
     * @return string
     */
    public function getAccountingType()
    {
        return $this->getConfigData('accounting');
    }

    /**
     *
     * @return int
     */
    public function oneClickEnabled()
    {
        return $this->getConfigData('oneclickenabled');
    }

    /**
     *
     * @return string
     */
    public function getUrl()
    {
        return ($this->paymentMethod->getConfigData('testmode') == 1) ? $this->gateway->URL_TEST : $this->gateway->URL_LIVE;
    }

    /**
     *
     * @return string
     */
    public function getAlias()
    {
        return $this->getConfigData('alias');
    }

    /**
     *
     * @return string
     */
    public function getMac()
    {
        return $this->getConfigData('mac');
    }

    /**
     *
     * @return string
     */
    public function getAliasOneClick()
    {
        return $this->getConfigData('alias_oneclick');
    }

    /**
     *
     * @return string
     */
    public function getMacOneClick()
    {
        return $this->getConfigData('mac_oneclick');
    }

    /**
     *
     * @return string
     */
    public function getUrlLogo()
    {
        return $this->getConfigData('url_logo');
    }

    /**
     *
     * @return bool
     */
    public function get3Dsecure20Enabled()
    {
        return $this->getConfigData('enable3dsecure20');
    }

    /**
     *
     * @return array
     */
    public function getAvailableMethods()
    {
        return json_decode($this->getConfigData('available_methods'), true);
    }
    
    /**
     * 
     * @return string
     */
    public function getInstallationUniqueId()
    {
        return $this->getConfigData('installation_unique_id');
    }

    /**
     *
     * @param string $param
     * @return string
     */
    private function getConfigData($param)
    {
        return trim($this->paymentMethod->getConfigData($param));
    }

}
