<?php

/**
 * Nexi XPay Payment Module
 *
 * @author      iPlusService S.r.l. <asssistenza@iplusservice.it>
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     4.0.0
 * @copyright   2017-2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */
class Iplusservice_XPay_Model_Payment extends Mage_Payment_Model_Method_Abstract
{

    protected $_infoBlockType = 'iplusservice_xpay/info';
    protected $_isGateway = true;
    protected $_canOrder = true;
    protected $_canRefund = true;
    protected $_canRefundInvoicePartial = true;

    /**
     *
     * @var Iplusservice_XPay_Model_Gateway
     */
    public $gateway;

    /**
     *
     * @var Iplusservice_XPay_Model_Configuration
     */
    public $configuration;

    public function __construct()
    {
        parent::__construct();

        $this->configuration = Mage::getModel('iplusservice_xpay/configuration', $this);

        $this->gateway = Mage::getModel('iplusservice_xpay/gateway');
    }

    public function refund(Varien_Object $payment, $amount)
    {
        $api = Mage::getModel('iplusservice_xpay/api', $this->configuration);

        if ($payment->getAdditionalInformation('alias') && $payment->getAdditionalInformation('transactionId')) {
            try {
                $api->refund(
                        $payment->getAdditionalInformation('alias'),
                        $payment->getAdditionalInformation('transactionId'),
                        $amount * 100
                );
            } catch (Exception $ex) {
                error_log($ex->getMessage() . ' ' . $ex->getTraceAsString());
                throw new Exception('Error during refund');
            }
        } else {
            throw new Exception('Error during refund');
        }

        return $this;
    }

    public function getConfigData($field, $storeId = null)
    {
        if (in_array($field, $this->getConfigFields())) {
            if (null === $storeId) {
                $storeId = $this->getStore();
            }

            $path = 'payment/iplusservice_xpay/' . $field;

            return Mage::getStoreConfig($path, $storeId);
        }

        return parent::getConfigData($field, $storeId);
    }

    private function getConfigFields()
    {
        return array(
            'active',
            'mac',
            'alias',
            'testmode',
            'alias_oneclick',
            'mac_oneclick',
            'available_methods',
            'language',
            'oneclickenabled',
            'accounting',
            'installation_unique_id'
        );
    }

    public function validate()
    {
        return $this;
    }

}
