<?php

/**
 * Nexi XPay Payment Module
 *
 * @author      iPlusService S.r.l. <asssistenza@iplusservice.it>
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     4.0.0
 * @copyright   2017-2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */
class Iplusservice_XPay_Model_Payment_Redirect extends Iplusservice_XPay_Model_Payment
{

    protected $_formBlockType = 'iplusservice_xpay/payment_redirect';
    public $_code = 'iplusservice_xpay';
    public $sortOrder = 1;
    public $title = 'Payment Cards';
    public $selectedCard = 'CC';

    public function getTitle()
    {
        return Mage::helper('iplusservice_xpay')->__($this->title);
    }

    public function getOrderPlaceRedirectUrl()
    {
        return Mage::getUrl('xpay/payment/redirect', array('_secure' => true, 'method' => $this->getCode()));
    }

    public function getUrlXPay()
    {
        $session = Mage::getSingleton('checkout/session');
        $order = Mage::getModel('sales/order');
        $order->loadByIncrementId($session->getLastRealOrderId());

        if (!$order->getId()) {
            error_log('Order not found');
            throw new Exception('Order not found.');
        }

        $amount = Mage::helper('iplusservice_xpay')->getGrandTotalFormatted($order);

        $alias = $this->configuration->getAlias();

        $transactionId = Mage::helper('iplusservice_xpay')->createTransactionId($order);

        $params = array(
            'alias' => $alias,
            'codTrans' => $transactionId,
            'importo' => $amount,
            'divisa' => 'EUR',
            'languageId' => Mage::helper('iplusservice_xpay')->getLanguageId($this),
            'urlpost' => Mage::getUrl('xpay/payment/notificas2s', array('_secure' => true)),
            'url' => Mage::getUrl('xpay/payment/conferma', array('_secure' => true)),
            'url_back' => Mage::getUrl('xpay/payment/annullo', array('_secure' => true)),
            'mail' => trim($order->getCustomerEmail()),
            'descrizione' => 'Ordine: ' . Mage::getModel('sales/quote')->load($order->getQuoteId())->getReservedOrderId(),
            'shipping' => 'N',
            'TCONTAB' => $this->configuration->getAccountingType(),
            'Note1' => 'magento',
            'Note2' => '1.9',
            'Note3' => Mage::helper('iplusservice_xpay')->pluginVersion()
        );

        if ($this->selectedCard) {
            $params['selectedcard'] = $this->selectedCard;
        }

        $payment = $order->getPayment();

        if ($this->configuration->get3Dsecure20Enabled()) {
            $params['Buyer_email'] = $order->getCustomerEmail();
            if ($order->getCustomerIsGuest() != 1) {
                $params['Buyer_account'] = $order->getCustomerEmail();

                $customerData = Mage::getModel('customer/customer')->load($order->getCustomerId());

                $now = new DateTime();

                $chAccDate = new \DateTime($customerData->getCreatedAt());
                $chAccDate_age_days = (int) ($chAccDate->diff($now)->format('%a'));
                if ($chAccDate_age_days < 30) {
                    $params['chAccAgeIndicator'] = "03";
                } elseif ($chAccDate_age_days < 60) {
                    $params['chAccAgeIndicator'] = "04";
                } else {
                    $params['chAccAgeIndicator'] = "05";
                }
                $params['chAccDate'] = substr($customerData->getCreatedAt(), 0, 10);

                $chAccChangeDate = new DateTime($customerData->getUpdatedAt());
                $chAccChangeDate_age_days = (int) ($chAccChangeDate->diff($now)->format('%a'));
                if ($chAccChangeDate_age_days < 30) {
                    $params['chAccChangeIndicator'] = "02";
                } elseif ($chAccChangeDate_age_days < 60) {
                    $params['chAccChangeIndicator'] = "03";
                } else {
                    $params['chAccChangeIndicator'] = "04";
                }
                $params['chAccChangeDate'] = substr($customerData->getUpdatedAt(), 0, 10);

                $orderCollection = Mage::getResourceModel('sales/order_collection')
                        ->addFieldToFilter('customer_id', $order->getCustomerId())
                        ->addAttributeToFilter('state', 'complete')
                        ->addAttributeToFilter('created_at', array('gteq' => date('Y-m-d', strtotime("-6 months"))));

                if ($orderCollection) {
                    $params['nbPurchaseAccount'] = count($orderCollection->getData());
                }
            }

            $cap_to_state_code = Mage::getModel('iplusservice_xpay/captostatecode');

            $shippingAddress = $order->getShippingAddress();

            if ($shippingAddress) {
                $shippingAddressCountryCode = $shippingAddress->getCountryId();

                $shippingAddressCountry = Mage::getModel('directory/country')->loadByCode($shippingAddressCountryCode);

                $params['Dest_city'] = $shippingAddress->getCity();
                $params['Dest_country'] = $shippingAddressCountry->getData('iso3_code');
                $params['Dest_street'] = implode("\n", $shippingAddress->getStreet());
                $params['Dest_cap'] = $shippingAddress->getPostcode();
                $params['Dest_state'] = $cap_to_state_code->getStateCode($shippingAddress->getPostcode());
            }

            $billingAddress = $order->getBillingAddress();

            if ($billingAddress) {
                $billingAddressCountryCode = $billingAddress->getCountryId();

                $billingAddressCountry = Mage::getModel('directory/country')->loadByCode($billingAddressCountryCode);
                
                $params['Bill_city'] = $billingAddress->getCity();
                $params['Bill_country'] = $billingAddressCountry->getData('iso3_code');
                $params['Bill_street'] = implode("\n", $billingAddress->getStreet());
                $params['Bill_cap'] = $billingAddress->getPostcode();
                $params['Bill_state'] = $cap_to_state_code->getStateCode($billingAddress->getPostcode());
            }
        }

        foreach ($params as $key => $value) {
            if (strlen($value) == 0) {
                unset($params[$key]);
            }
        }

        $payment->setAdditionalInformation('alias', $alias);
        $payment->setAdditionalInformation('transactionId', $transactionId);
        $payment->setTransactionId($transactionId);
        $payment->save();

        $customerId = Mage::getSingleton('customer/session')->getCustomerId();

        if (
                $customerId &&
                Mage::getSingleton('customer/session')->isLoggedIn() &&
                $this->configuration->oneClickEnabled() &&
                $this->getCode() == 'iplusservice_xpay' &&
                $this->configuration->getInstallationUniqueId()
        ) {
            $params['num_contratto'] = $this->configuration->getInstallationUniqueId() . '-' . $customerId;
            $params['tipo_servizio'] = 'paga_1click';

            $params['mac'] = sha1('codTrans=' . $transactionId . 'divisa=EURimporto=' . $amount . 'gruppo=num_contratto=' . $params['num_contratto'] . $this->configuration->getMac());
        } else {
            $params['mac'] = sha1('codTrans=' . $transactionId . 'divisa=EURimporto=' . $amount . $this->configuration->getMac());
        }

        return array(
            "target" => $this->configuration->getUrl() . $this->gateway->URI_REDIRECT,
            "params" => $params,
        );
    }

    /**
     * 
     * @param type $quote
     * @return type
     */
    public function isAvailable($quote = null)
    {
        // se è un apm deve essere attivo l'apm e non dev'essere nella parte admin
        $apmCondition = $this->apmName && $this->isAlternativeMethodAvailable($this->apmName) && !Mage::app()->getStore()->isAdmin();

        // se è il redirect facciamo fede alla funzione padre per vedere se è disponibile o meno
        $redirectCondition = !$this->apmName;

        return parent::isAvailable($quote) && ( $apmCondition || $redirectCondition );
    }

    private function isAlternativeMethodAvailable($alternativeMethod)
    {
        foreach ($this->configuration->getAvailableMethods() as $availableMethod) {
            if (strtolower(trim($availableMethod['code'])) == $alternativeMethod) {
                return true;
            }
        }

        return false;
    }

}
