<?php

/**
 * Nexi XPay Payment Module
 *
 * @author      iPlusService S.r.l. <asssistenza@iplusservice.it>
 * @category    Payment Module
 * @package     Nexi XPay
 * @version     4.0.1
 * @copyright   2017-2019 Nexi Payments S.p.A. (https://ecommerce.nexi.it)
 * @license     GNU General Public License v3.0
 */
class Iplusservice_XPay_PaymentController extends Mage_Core_Controller_Front_Action
{

    public function redirectAction()
    {
        $method = $this->getRequest()->get('method');

        $model = 'iplusservice_xpay/payment_redirect' . substr($method, strlen('iplusservice_xpay'));
        
        $forminfo = Mage::getModel($model)->getUrlXPay();

        $this->getResponse()->setBody($this->getLayout()->createBlock('core/template')->setData("forminfo", $forminfo)->setTemplate("iplusservice/xpay/autosubmit/form.phtml", $data)->toHtml());
    }

    public function notificas2sAction()
    {
        $errors = array();

        $requiredFields = array('codTrans', 'esito', 'importo', 'divisa', 'data', 'orario', 'codAut', 'mac');
        foreach ($requiredFields as $field) {
            if ($this->getRequest()->getPost($field, 'vuoto') == 'vuoto') {
                $errors[] = 'Missing field ' . $field;
                error_log('Missing field ' . $field);
                header('HTTP/1.1 500 Internal Server Error');
            }
        }

        if (count($errors) == 0) {
            $configuration = Mage::getModel('iplusservice_xpay/configuration', Mage::getModel('iplusservice_xpay/payment_redirect'));

            $macComputed = sha1(
                    'codTrans=' . $this->getRequest()->getPost('codTrans')
                    . 'esito=' . $this->getRequest()->getPost('esito')
                    . 'importo=' . $this->getRequest()->getPost('importo')
                    . 'divisa=' . $this->getRequest()->getPost('divisa')
                    . 'data=' . $this->getRequest()->getPost('data')
                    . 'orario=' . $this->getRequest()->getPost('orario')
                    . 'codAut=' . $this->getRequest()->getPost('codAut')
                    . $configuration->getMac()
            );

            if ($macComputed != $this->getRequest()->getPost('mac')) {
                Mage::log('S2S errore MAC: ' . $macComputed . ' != ' . $this->getRequest()->getPost('mac'), null, 'xpay.log');
                error_log('Mac error ' . $field);
                header('HTTP/1.1 500 Internal Server Error');
            } else {
                Mage::log('Transaction Result: ' . $this->getRequest()->getPost('esito'), null, 'xpay.log');

                if ($this->getRequest()->getPost('esito') == 'OK') {
                    $statoOrdine = Mage_Sales_Model_Order::STATE_PROCESSING;

                    $orderId = explode('-', $this->getRequest()->getPost('codTrans'))[0];

                    // Check order
                    $order = Mage::getModel('sales/order')->load($orderId);

                    if (!$order) {
                        Mage::log('Order not found orderId: ' . $orderId, null, 'xpay.log');
                        error_Log('Order not found orderId: ' . $orderId);
                        header('HTTP/1.1 500 Internal Server Error');
                    } else {
                        $payment = $order->getPayment();

                        // Save order status
                        $order->setStatus($statoOrdine);
                        $order->setState($statoOrdine);

                        $order->addStatusToHistory($statoOrdine, '', false);

                        // Create invoice
                        Mage::helper('iplusservice_xpay')->invoiceOrder($order, $this->getRequest()->getPost('codTrans'));

                        if (!$payment->hasAdditionalInformation('paymail') || $payment->getAdditionalInformation('paymail') != 1) {
                            // Notify order
                            $order->sendNewOrderEmail();
                            $order->setEmailSent(true);
                        }

                        // Save order
                        $order->save();
                    }
                } else {
                    Mage::log('S2S Transaction KO - codTrans: ' . $this->getRequest()->getPost('codTrans'), null, 'xpay.log');
                }
            }
        }
    }

    public function confermaAction()
    {

        $orderId = explode('-', $this->getRequest()->getParam('codTrans'))[0];

        if ($this->getRequest()->getQuery('esito') == 'OK') {
            // Check order
            if (!$orderId) {
                header('Location: ' . Mage::helper('core/url')->getCurrentUrl());
            }

            $order = Mage::getModel('sales/order')->load($orderId);

            $session = Mage::getSingleton('checkout/session');
            $session->clearHelperData();
            $session->setLastQuoteId($order->getQuoteId())->setLastSuccessQuoteId($order->getQuoteId());
            $session->setLastOrderId($orderId)->setLastRealOrderId($order->getIncrementId());

            $this->_redirect('checkout/onepage/success', array('_secure' => true));
        } else {
            // Cart reset
            $checkoutSession = Mage::getSingleton('checkout/session');

            if ($lastQuoteId = $checkoutSession->getLastQuoteId()) {
                $quote = Mage::getModel('sales/quote')->load($lastQuoteId);
                if ($quoteId = $quote->getId()) {
                    $quote->setIsActive(true);
                    $quote->setReservedOrderId(null);
                    $quote->save();
                    $checkoutSession->setQuoteId($quoteId);
                }
            }

            $checkoutSession->addError($this->__('Error during checkout.'));

            $this->_redirect('checkout/cart', array('_secure' => true));
        }
    }

    public function annulloAction()
    {
        $checkoutSession = Mage::getSingleton('checkout/session');

        if ($lastQuoteId = $checkoutSession->getLastQuoteId()) {
            $quote = Mage::getModel('sales/quote')->load($lastQuoteId);

            $orderId = $checkoutSession->getLastOrderId();
            $order = Mage::getModel('sales/order')->load($orderId);
            $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true)->save();

            if ($quoteId = $quote->getId()) {
                $quote->setIsActive(true);
                $quote->setReservedOrderId(null);
                $quote->save();
                $checkoutSession->setQuoteId($quoteId);
            }
        }

        $checkoutSession->addError($this->__('Order have been canceled.'));
        $this->_redirect('checkout/cart', array('_secure' => true));
    }

}
